module Targetmy
  class Config
    class Site
      class Teaser
        attr_reader :type, :campaign_name, :ads
        
        SUPPORTED_KEYS = [:type, :campaign_name, :ads]
        SUPPORTED_ADS_KEYS = [:title, :text, :image]
        REQUIRED_KEYS = [:ads, :type, :campaign_name]
        REQUIRED_ADS_KEYS = [:title, :text, :image]

        def initialize(config)
          parse config
        end

        def parse(config)
          config_required_items = REQUIRED_KEYS & config.keys
          raise Targetmy::MissingRequiredKeys if config_required_items.length != REQUIRED_KEYS.length

          SUPPORTED_KEYS.each do |key|
            next unless config.has_key?(key)

            case key
            when :ads
              @ads = []
              parse_ads config[key]
            else
              instance_variable_set("@#{key}", config[key])
            end
          end
        end

        def parse_ads(ads)
          ads = ads.is_a?(Hash) ? [ads] : ads

          ads.each do |ad|
            ad_obj = OpenStruct.new

            ad_required_items = REQUIRED_ADS_KEYS & ad.keys
            raise Targetmy::MissingRequiredKeys if ad_required_items.length != REQUIRED_ADS_KEYS.length

            SUPPORTED_ADS_KEYS.each do |key|
              next unless ad.has_key?(key)

              ad_obj["#{key}"] = ad[key]
            end

            @ads << ad_obj
          end
        end
        
      end
    end
  end
end